@@ -15,6 +15,13 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod codec;
 mod list;
 mod map;
 pub mod row;
+mod shuffle_writer;
+pub use codec::BatchWriter;
+
+pub use shuffle_writer::{
+    read_ipc_compressed, CompressionCodec, ShuffleBlockWriter, ShuffleWriterExec,
+};
