@@ -25,11 +25,12 @@ import org.apache.spark.SparkConf
 import org.apache.spark.internal.Logging
 import org.apache.spark.network.util.ByteUnit
 import org.apache.spark.sql.{SparkSession, SparkSessionExtensions}
-import org.apache.spark.sql.catalyst.expressions.{Divide, DoubleLiteral, EqualNullSafe, EqualTo, Expression, FloatLiteral, GreaterThan, GreaterThanOrEqual, KnownFloatingPointNormalized, LessThan, LessThanOrEqual, NamedExpression, PlanExpression, Remainder}
+import org.apache.spark.sql.catalyst.expressions.{Attribute, Divide, DoubleLiteral, EqualNullSafe, EqualTo, Expression, FloatLiteral, GreaterThan, GreaterThanOrEqual, KnownFloatingPointNormalized, LessThan, LessThanOrEqual, NamedExpression, PlanExpression, Remainder}
 import org.apache.spark.sql.catalyst.expressions.aggregate.{Final, Partial}
 import org.apache.spark.sql.catalyst.optimizer.NormalizeNaNAndZero
 import org.apache.spark.sql.catalyst.rules.Rule
 import org.apache.spark.sql.catalyst.trees.TreeNode
+import org.apache.spark.sql.catalyst.util.MetadataColumnHelper
 import org.apache.spark.sql.comet._
 import org.apache.spark.sql.comet.execution.shuffle.{CometColumnarShuffle, CometNativeShuffle, CometShuffleExchangeExec, CometShuffleManager}
 import org.apache.spark.sql.comet.util.Utils
@@ -54,6 +55,7 @@ import org.apache.comet.CometConf._
 import org.apache.comet.CometExplainInfo.getActualPlan
 import org.apache.comet.CometSparkSessionExtensions.{createMessage, getCometBroadcastNotEnabledReason, getCometShuffleNotEnabledReason, isANSIEnabled, isCometBroadCastForceEnabled, isCometEnabled, isCometExecEnabled, isCometJVMShuffleMode, isCometNativeShuffleMode, isCometScan, isCometScanEnabled, isCometShuffleEnabled, isSpark34Plus, isSpark40Plus, shouldApplySparkToColumnar, withInfo, withInfos}
 import org.apache.comet.parquet.{CometParquetScan, SupportsComet}
+import org.apache.comet.rules.RewriteJoin
 import org.apache.comet.serde.OperatorOuterClass.Operator
 import org.apache.comet.serde.QueryPlanSerde
 import org.apache.comet.shims.ShimCometSparkSessionExtensions
@@ -101,7 +103,19 @@ class CometSparkSessionExtensions
         def isDynamicPruningFilter(e: Expression): Boolean =
           e.exists(_.isInstanceOf[PlanExpression[_]])
 
+        def hasMetadataCol(plan: SparkPlan): Boolean = {
+          plan.expressions.exists(_.exists {
+            case a: Attribute =>
+              a.isMetadataCol
+            case _ => false
+          })
+        }
+
         plan.transform {
+          case scan if hasMetadataCol(scan) =>
+            withInfo(scan, "Metadata column is not supported")
+            scan
+
           case scanExec: FileSourceScanExec
               if COMET_DPP_FALLBACK_ENABLED.get() &&
                 scanExec.partitionFilters.exists(isDynamicPruningFilter) =>
@@ -176,7 +190,7 @@ class CometSparkSessionExtensions
 
           // data source V1
           case scanExec @ FileSourceScanExec(
-                HadoopFsRelation(_, partitionSchema, _, _, _: ParquetFileFormat, _),
+                HadoopFsRelation(_, partitionSchema, _, _, fileFormat, _),
                 _: Seq[_],
                 requiredSchema,
                 _,
@@ -185,14 +199,38 @@ class CometSparkSessionExtensions
                 _,
                 _,
                 _)
-              if CometScanExec.isSchemaSupported(requiredSchema)
+              if CometScanExec.isFileFormatSupported(fileFormat)
+                && CometNativeScanExec.isSchemaSupported(requiredSchema)
+                && CometNativeScanExec.isSchemaSupported(partitionSchema)
+                // TODO we only enable full native scan if COMET_EXEC_ENABLED is enabled
+                // but this is not really what we want .. we currently insert `CometScanExec`
+                // here and then it gets replaced with `CometNativeScanExec` in `CometExecRule`
+                // but that only happens if `COMET_EXEC_ENABLED` is enabled
+                && COMET_EXEC_ENABLED.get()
+                && COMET_NATIVE_SCAN_IMPL.get() == CometConf.SCAN_NATIVE_DATAFUSION =>
+            logInfo("Comet extension enabled for v1 full native Scan")
+            CometScanExec(scanExec, session)
+
+          // data source V1
+          case scanExec @ FileSourceScanExec(
+                HadoopFsRelation(_, partitionSchema, _, _, fileFormat, _),
+                _: Seq[_],
+                requiredSchema,
+                _,
+                _,
+                _,
+                _,
+                _,
+                _)
+              if CometScanExec.isFileFormatSupported(fileFormat)
+                && CometScanExec.isSchemaSupported(requiredSchema)
                 && CometScanExec.isSchemaSupported(partitionSchema) =>
             logInfo("Comet extension enabled for v1 Scan")
             CometScanExec(scanExec, session)
 
           // data source v1 not supported case
           case scanExec @ FileSourceScanExec(
-                HadoopFsRelation(_, partitionSchema, _, _, _: ParquetFileFormat, _),
+                HadoopFsRelation(_, partitionSchema, _, _, fileFormat, _),
                 _: Seq[_],
                 requiredSchema,
                 _,
@@ -202,12 +240,15 @@ class CometSparkSessionExtensions
                 _,
                 _) =>
             val info1 = createMessage(
+              !CometScanExec.isFileFormatSupported(fileFormat),
+              s"File format $fileFormat is not supported")
+            val info2 = createMessage(
               !CometScanExec.isSchemaSupported(requiredSchema),
               s"Schema $requiredSchema is not supported")
-            val info2 = createMessage(
+            val info3 = createMessage(
               !CometScanExec.isSchemaSupported(partitionSchema),
               s"Partition schema $partitionSchema is not supported")
-            withInfo(scanExec, Seq(info1, info2).flatten.mkString(","))
+            withInfo(scanExec, Seq(info1, info2, info3).flatten.mkString(","))
             scanExec
         }
       }
@@ -335,6 +376,13 @@ class CometSparkSessionExtensions
       }
 
       plan.transformUp {
+        // Fully native scan for V1
+        case scan: CometScanExec
+            if COMET_NATIVE_SCAN_IMPL.get().equals(CometConf.SCAN_NATIVE_DATAFUSION) =>
+          val nativeOp = QueryPlanSerde.operator2Proto(scan).get
+          CometNativeScanExec(nativeOp, scan.wrapped, scan.session)
+
+        // Comet JVM + native scan for V1 and V2
         case op if isCometScan(op) =>
           val nativeOp = QueryPlanSerde.operator2Proto(op).get
           CometScanWrapper(nativeOp, op)
@@ -439,7 +487,11 @@ class CometSparkSessionExtensions
 
         case op: BaseAggregateExec
             if op.isInstanceOf[HashAggregateExec] ||
-              op.isInstanceOf[ObjectHashAggregateExec] =>
+              op.isInstanceOf[ObjectHashAggregateExec] &&
+              // When Comet shuffle is disabled, we don't want to transform the HashAggregate
+              // to CometHashAggregate. Otherwise, we probably get partial Comet aggregation
+              // and final Spark aggregation.
+              isCometShuffleEnabled(conf) =>
           val groupingExprs = op.groupingExpressions
           val aggExprs = op.aggregateExpressions
           val resultExpressions = op.resultExpressions
@@ -451,8 +503,10 @@ class CometSparkSessionExtensions
             // Fallback to Spark nevertheless here.
             op
           } else {
+            // For a final mode HashAggregate, we only need to transform the HashAggregate
+            // if there is Comet partial aggregation.
             val sparkFinalMode = {
-              !modes.isEmpty && modes.head == Final && findPartialAgg(child).isEmpty
+              !modes.isEmpty && modes.head == Final && findCometPartialAgg(child).isEmpty
             }
 
             if (sparkFinalMode) {
@@ -543,7 +597,7 @@ class CometSparkSessionExtensions
 
         case op: SortMergeJoinExec
             if CometConf.COMET_EXEC_SORT_MERGE_JOIN_ENABLED.get(conf) &&
-              op.children.forall(isCometNative(_)) =>
+              op.children.forall(isCometNative) =>
           val newOp = transform1(op)
           newOp match {
             case Some(nativeOp) =>
@@ -764,68 +818,93 @@ class CometSparkSessionExtensions
           }
 
         // Native shuffle for Comet operators
-        case s: ShuffleExchangeExec
-            if isCometShuffleEnabled(conf) &&
-              isCometNativeShuffleMode(conf) &&
-              QueryPlanSerde.supportPartitioning(s.child.output, s.outputPartitioning)._1 =>
-          logInfo("Comet extension enabled for Native Shuffle")
-
-          val newOp = transform1(s)
-          newOp match {
-            case Some(nativeOp) =>
-              // Switch to use Decimal128 regardless of precision, since Arrow native execution
-              // doesn't support Decimal32 and Decimal64 yet.
-              conf.setConfString(CometConf.COMET_USE_DECIMAL_128.key, "true")
-              val cometOp = CometShuffleExchangeExec(s, shuffleType = CometNativeShuffle)
-              CometSinkPlaceHolder(nativeOp, s, cometOp)
-            case None =>
-              s
-          }
-
-        // Columnar shuffle for regular Spark operators (not Comet) and Comet operators
-        // (if configured).
-        // If the child of ShuffleExchangeExec is also a ShuffleExchangeExec, we should not
-        // convert it to CometColumnarShuffle,
-        case s: ShuffleExchangeExec
-            if isCometShuffleEnabled(conf) && isCometJVMShuffleMode(conf) &&
-              QueryPlanSerde.supportPartitioningTypes(s.child.output, s.outputPartitioning)._1 &&
-              !isShuffleOperator(s.child) =>
-          logInfo("Comet extension enabled for JVM Columnar Shuffle")
-
-          val newOp = QueryPlanSerde.operator2Proto(s)
-          newOp match {
-            case Some(nativeOp) =>
-              s.child match {
-                case n if n.isInstanceOf[CometNativeExec] || !n.supportsColumnar =>
-                  val cometOp = CometShuffleExchangeExec(s, shuffleType = CometColumnarShuffle)
-                  CometSinkPlaceHolder(nativeOp, s, cometOp)
-                case _ =>
-                  s
-              }
-            case None =>
-              s
-          }
-
         case s: ShuffleExchangeExec =>
-          val isShuffleEnabled = isCometShuffleEnabled(conf)
-          val outputPartitioning = s.outputPartitioning
-          val reason = getCometShuffleNotEnabledReason(conf).getOrElse("no reason available")
-          val msg1 = createMessage(!isShuffleEnabled, s"Comet shuffle is not enabled: $reason")
-          val columnarShuffleEnabled = isCometJVMShuffleMode(conf)
-          val msg2 = createMessage(
-            isShuffleEnabled && !columnarShuffleEnabled && !QueryPlanSerde
-              .supportPartitioning(s.child.output, outputPartitioning)
-              ._1,
-            "Native shuffle: " +
-              s"${QueryPlanSerde.supportPartitioning(s.child.output, outputPartitioning)._2}")
-          val msg3 = createMessage(
-            isShuffleEnabled && columnarShuffleEnabled && !QueryPlanSerde
+          val nativePrecondition = isCometShuffleEnabled(conf) &&
+            isCometNativeShuffleMode(conf) &&
+            QueryPlanSerde.supportPartitioning(s.child.output, s.outputPartitioning)._1
+
+          val nativeShuffle: Option[SparkPlan] =
+            if (nativePrecondition) {
+              val newOp = transform1(s)
+              newOp match {
+                case Some(nativeOp) =>
+                  // Switch to use Decimal128 regardless of precision, since Arrow native execution
+                  // doesn't support Decimal32 and Decimal64 yet.
+                  conf.setConfString(CometConf.COMET_USE_DECIMAL_128.key, "true")
+                  val cometOp = CometShuffleExchangeExec(s, shuffleType = CometNativeShuffle)
+                  Some(CometSinkPlaceHolder(nativeOp, s, cometOp))
+                case None =>
+                  None
+              }
+            } else {
+              None
+            }
+
+          // this is a temporary workaround because some Spark SQL tests fail
+          // when we enable COMET_SHUFFLE_FALLBACK_TO_COLUMNAR due to valid bugs
+          // that we had not previously seen
+          val tryColumnarNext =
+            !nativePrecondition || (nativePrecondition && nativeShuffle.isEmpty &&
+              COMET_SHUFFLE_FALLBACK_TO_COLUMNAR.get(conf))
+
+          val nativeOrColumnarShuffle = if (nativeShuffle.isDefined) {
+            nativeShuffle
+          } else if (tryColumnarNext) {
+            // Columnar shuffle for regular Spark operators (not Comet) and Comet operators
+            // (if configured).
+            // If the child of ShuffleExchangeExec is also a ShuffleExchangeExec, we should not
+            // convert it to CometColumnarShuffle,
+            if (isCometShuffleEnabled(conf) && isCometJVMShuffleMode(conf) &&
+              QueryPlanSerde.supportPartitioningTypes(s.child.output, s.outputPartitioning)._1 &&
+              !isShuffleOperator(s.child)) {
+
+              val newOp = QueryPlanSerde.operator2Proto(s)
+              newOp match {
+                case Some(nativeOp) =>
+                  s.child match {
+                    case n if n.isInstanceOf[CometNativeExec] || !n.supportsColumnar =>
+                      val cometOp =
+                        CometShuffleExchangeExec(s, shuffleType = CometColumnarShuffle)
+                      Some(CometSinkPlaceHolder(nativeOp, s, cometOp))
+                    case _ =>
+                      None
+                  }
+                case None =>
+                  None
+              }
+            } else {
+              None
+            }
+          } else {
+            None
+          }
+
+          if (nativeOrColumnarShuffle.isDefined) {
+            nativeOrColumnarShuffle.get
+          } else {
+            val isShuffleEnabled = isCometShuffleEnabled(conf)
+            val outputPartitioning = s.outputPartitioning
+            val reason = getCometShuffleNotEnabledReason(conf).getOrElse("no reason available")
+            val msg1 = createMessage(!isShuffleEnabled, s"Comet shuffle is not enabled: $reason")
+            val columnarShuffleEnabled = isCometJVMShuffleMode(conf)
+            val msg2 = createMessage(
+              isShuffleEnabled && !columnarShuffleEnabled && !QueryPlanSerde
+                .supportPartitioning(s.child.output, outputPartitioning)
+                ._1,
+              "Native shuffle: " +
+                s"${QueryPlanSerde.supportPartitioning(s.child.output, outputPartitioning)._2}")
+            val typeInfo = QueryPlanSerde
               .supportPartitioningTypes(s.child.output, outputPartitioning)
-              ._1,
-            "JVM shuffle: " +
-              s"${QueryPlanSerde.supportPartitioningTypes(s.child.output, outputPartitioning)._2}")
-          withInfo(s, Seq(msg1, msg2, msg3).flatten.mkString(","))
-          s
+              ._2
+            val msg3 = createMessage(
+              isShuffleEnabled && columnarShuffleEnabled && !QueryPlanSerde
+                .supportPartitioningTypes(s.child.output, outputPartitioning)
+                ._1,
+              "JVM shuffle: " +
+                s"$typeInfo")
+            withInfo(s, Seq(msg1, msg2, msg3).flatten.mkString(","))
+            s
+          }
 
         case op =>
           op match {
@@ -919,7 +998,15 @@ class CometSparkSessionExtensions
           plan
         }
       } else {
-        var newPlan = transform(normalizePlan(plan))
+        val normalizedPlan = if (CometConf.COMET_REPLACE_SMJ.get()) {
+          normalizePlan(plan).transformUp { case p =>
+            RewriteJoin.rewrite(p)
+          }
+        } else {
+          normalizePlan(plan)
+        }
+
+        var newPlan = transform(normalizedPlan)
 
         // if the plan cannot be run fully natively then explain why (when appropriate
         // config is enabled)
@@ -978,12 +1065,20 @@ class CometSparkSessionExtensions
         var firstNativeOp = true
         newPlan.transformDown {
           case op: CometNativeExec =>
-            if (firstNativeOp) {
+            val newPlan = if (firstNativeOp) {
               firstNativeOp = false
               op.convertBlock()
             } else {
               op
             }
+
+            // If reaching leaf node, reset `firstNativeOp` to true
+            // because it will start a new block in next iteration.
+            if (op.children.isEmpty) {
+              firstNativeOp = true
+            }
+
+            newPlan
           case op =>
             firstNativeOp = true
             op
@@ -995,13 +1090,15 @@ class CometSparkSessionExtensions
      * Find the first Comet partial aggregate in the plan. If it reaches a Spark HashAggregate
      * with partial mode, it will return None.
      */
-    def findPartialAgg(plan: SparkPlan): Option[CometHashAggregateExec] = {
+    def findCometPartialAgg(plan: SparkPlan): Option[CometHashAggregateExec] = {
       plan.collectFirst {
         case agg: CometHashAggregateExec if agg.aggregateExpressions.forall(_.mode == Partial) =>
           Some(agg)
         case agg: HashAggregateExec if agg.aggregateExpressions.forall(_.mode == Partial) => None
-        case a: AQEShuffleReadExec => findPartialAgg(a.child)
-        case s: ShuffleQueryStageExec => findPartialAgg(s.plan)
+        case agg: ObjectHashAggregateExec if agg.aggregateExpressions.forall(_.mode == Partial) =>
+          None
+        case a: AQEShuffleReadExec => findCometPartialAgg(a.child)
+        case s: ShuffleQueryStageExec => findCometPartialAgg(s.plan)
       }.flatten
     }
 
@@ -1043,6 +1140,17 @@ class CometSparkSessionExtensions
     override def apply(plan: SparkPlan): SparkPlan = {
       val eliminatedPlan = plan transformUp {
         case ColumnarToRowExec(sparkToColumnar: CometSparkToColumnarExec) => sparkToColumnar.child
+        case c @ ColumnarToRowExec(child) if child.exists(_.isInstanceOf[CometPlan]) =>
+          val op = CometColumnarToRowExec(child)
+          if (c.logicalLink.isEmpty) {
+            op.unsetTagValue(SparkPlan.LOGICAL_PLAN_TAG)
+            op.unsetTagValue(SparkPlan.LOGICAL_PLAN_INHERITED_TAG)
+          } else {
+            c.logicalLink.foreach(op.setLogicalLink)
+          }
+          op
+        case CometColumnarToRowExec(sparkToColumnar: CometSparkToColumnarExec) =>
+          sparkToColumnar.child
         case CometSparkToColumnarExec(child: CometSparkToColumnarExec) => child
         // Spark adds `RowToColumnar` under Comet columnar shuffle. But it's redundant as the
         // shuffle takes row-based input.
@@ -1059,6 +1167,8 @@ class CometSparkSessionExtensions
       eliminatedPlan match {
         case ColumnarToRowExec(child: CometCollectLimitExec) =>
           child
+        case CometColumnarToRowExec(child: CometCollectLimitExec) =>
+          child
         case other =>
           other
       }
@@ -1132,6 +1242,10 @@ object CometSparkSessionExtensions extends Logging {
     }
   }
 
+  // Check whether Comet shuffle is enabled:
+  // 1. `COMET_EXEC_SHUFFLE_ENABLED` is true
+  // 2. `spark.shuffle.manager` is set to `CometShuffleManager`
+  // 3. Off-heap memory is enabled || Spark/Comet unit testing
   private[comet] def isCometShuffleEnabled(conf: SQLConf): Boolean =
     COMET_EXEC_SHUFFLE_ENABLED.get(conf) && isCometShuffleManagerEnabled(conf)
 
@@ -1238,21 +1352,41 @@ object CometSparkSessionExtensions extends Logging {
     org.apache.spark.SPARK_VERSION >= "4.0"
   }
 
+  def isComplexTypeReaderEnabled(conf: SQLConf): Boolean = {
+    CometConf.COMET_NATIVE_SCAN_IMPL.get(conf) == CometConf.SCAN_NATIVE_ICEBERG_COMPAT ||
+    CometConf.COMET_NATIVE_SCAN_IMPL.get(conf) == CometConf.SCAN_NATIVE_DATAFUSION
+  }
+
+  def usingDataFusionParquetReader(conf: SQLConf): Boolean = {
+    isComplexTypeReaderEnabled(conf) && !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)
+  }
+
   /** Calculates required memory overhead in MB per executor process for Comet. */
   def getCometMemoryOverheadInMiB(sparkConf: SparkConf): Long = {
-    // `spark.executor.memory` default value is 1g
-    val executorMemoryMiB = ConfigHelpers
-      .byteFromString(sparkConf.get("spark.executor.memory", "1024MB"), ByteUnit.MiB)
+    val baseMemoryMiB = if (cometUnifiedMemoryManagerEnabled(sparkConf)) {
+      ConfigHelpers
+        .byteFromString(sparkConf.get("spark.memory.offHeap.size"), ByteUnit.MiB)
+    } else {
+      // `spark.executor.memory` default value is 1g
+      ConfigHelpers
+        .byteFromString(sparkConf.get("spark.executor.memory", "1024MB"), ByteUnit.MiB)
+    }
 
     val minimum = ConfigHelpers
-      .byteFromString(sparkConf.get(COMET_MEMORY_OVERHEAD_MIN_MIB.key, "384"), ByteUnit.MiB)
-    val overheadFactor = sparkConf.getDouble(COMET_MEMORY_OVERHEAD_FACTOR.key, 0.2)
+      .byteFromString(
+        sparkConf.get(
+          COMET_MEMORY_OVERHEAD_MIN_MIB.key,
+          COMET_MEMORY_OVERHEAD_MIN_MIB.defaultValueString),
+        ByteUnit.MiB)
+    val overheadFactor = sparkConf.getDouble(
+      COMET_MEMORY_OVERHEAD_FACTOR.key,
+      COMET_MEMORY_OVERHEAD_FACTOR.defaultValue.get)
 
     val overHeadMemFromConf = sparkConf
       .getOption(COMET_MEMORY_OVERHEAD.key)
       .map(ConfigHelpers.byteFromString(_, ByteUnit.MiB))
 
-    overHeadMemFromConf.getOrElse(math.max((overheadFactor * executorMemoryMiB).toLong, minimum))
+    overHeadMemFromConf.getOrElse(math.max((overheadFactor * baseMemoryMiB).toLong, minimum))
   }
 
   /** Calculates required memory overhead in bytes per executor process for Comet. */
@@ -1261,7 +1395,7 @@ object CometSparkSessionExtensions extends Logging {
   }
 
   /** Calculates required shuffle memory size in bytes per executor process for Comet. */
-  def getCometShuffleMemorySize(sparkConf: SparkConf, conf: SQLConf): Long = {
+  def getCometShuffleMemorySize(sparkConf: SparkConf, conf: SQLConf = SQLConf.get): Long = {
     val cometMemoryOverhead = getCometMemoryOverheadInMiB(sparkConf)
 
     val overheadFactor = COMET_COLUMNAR_SHUFFLE_MEMORY_FACTOR.get(conf)
@@ -1279,6 +1413,16 @@ object CometSparkSessionExtensions extends Logging {
     }
   }
 
+  def cometUnifiedMemoryManagerEnabled(sparkConf: SparkConf): Boolean = {
+    sparkConf.getBoolean("spark.memory.offHeap.enabled", false)
+  }
+
+  def cometShuffleUnifiedMemoryManagerInTestEnabled(sparkConf: SparkConf): Boolean = {
+    sparkConf.getBoolean(
+      CometConf.COMET_COLUMNAR_SHUFFLE_UNIFIED_MEMORY_ALLOCATOR_IN_TEST.key,
+      CometConf.COMET_COLUMNAR_SHUFFLE_UNIFIED_MEMORY_ALLOCATOR_IN_TEST.defaultValue.get)
+  }
+
   /**
    * Attaches explain information to a TreeNode, rolling up the corresponding information tags
    * from any child nodes. For now, we are using this to attach the reasons why certain Spark
