@@ -36,14 +36,13 @@ import org.apache.parquet.hadoop.example.ExampleParquetWriter
 import org.apache.parquet.schema.{MessageType, MessageTypeParser}
 import org.apache.spark._
 import org.apache.spark.internal.config.{MEMORY_OFFHEAP_ENABLED, MEMORY_OFFHEAP_SIZE, SHUFFLE_MANAGER}
-import org.apache.spark.sql.comet.{CometBatchScanExec, CometBroadcastExchangeExec, CometExec, CometScanExec, CometScanWrapper, CometSinkPlaceHolder, CometSparkToColumnarExec}
+import org.apache.spark.sql.comet._
 import org.apache.spark.sql.comet.execution.shuffle.{CometColumnarShuffle, CometNativeShuffle, CometShuffleExchangeExec}
-import org.apache.spark.sql.execution.{ColumnarToRowExec, ExtendedMode, InputAdapter, SparkPlan, WholeStageCodegenExec}
+import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.internal._
 import org.apache.spark.sql.test._
-import org.apache.spark.sql.types.DecimalType
-import org.apache.spark.sql.types.StructType
+import org.apache.spark.sql.types.{DecimalType, StructType}
 
 import org.apache.comet._
 import org.apache.comet.CometSparkSessionExtensions.isSpark34Plus
@@ -78,7 +77,9 @@ abstract class CometTestBase
     conf.set(CometConf.COMET_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_SHUFFLE_ENABLED.key, "true")
+    conf.set(CometConf.COMET_SHUFFLE_FALLBACK_TO_COLUMNAR.key, "true")
     conf.set(CometConf.COMET_SPARK_TO_ARROW_ENABLED.key, "true")
+    conf.set(CometConf.COMET_NATIVE_SCAN_ENABLED.key, "true")
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "2g")
     conf.set(CometConf.COMET_EXEC_SORT_MERGE_JOIN_WITH_JOIN_FILTER_ENABLED.key, "true")
     conf
@@ -172,8 +173,9 @@ abstract class CometTestBase
   protected def checkCometOperators(plan: SparkPlan, excludedClasses: Class[_]*): Unit = {
     val wrapped = wrapCometSparkToColumnar(plan)
     wrapped.foreach {
-      case _: CometScanExec | _: CometBatchScanExec =>
+      case _: CometNativeScanExec | _: CometScanExec | _: CometBatchScanExec =>
       case _: CometSinkPlaceHolder | _: CometScanWrapper =>
+      case _: CometColumnarToRowExec =>
       case _: CometSparkToColumnarExec =>
       case _: CometExec | _: CometShuffleExchangeExec =>
       case _: CometBroadcastExchangeExec =>
@@ -231,17 +233,16 @@ abstract class CometTestBase
       df: => DataFrame): (Option[Throwable], Option[Throwable]) = {
     var expected: Option[Throwable] = None
     withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
-      val dfSpark = Dataset.ofRows(spark, df.logicalPlan)
-      expected = Try(dfSpark.collect()).failed.toOption
+      expected = Try(Dataset.ofRows(spark, df.logicalPlan).collect()).failed.toOption
     }
-    val dfComet = Dataset.ofRows(spark, df.logicalPlan)
-    val actual = Try(dfComet.collect()).failed.toOption
+    val actual = Try(Dataset.ofRows(spark, df.logicalPlan).collect()).failed.toOption
     (expected, actual)
   }
 
   protected def checkSparkAnswerAndCompareExplainPlan(
       df: DataFrame,
-      expectedInfo: Set[String]): Unit = {
+      expectedInfo: Set[String],
+      checkExplainString: Boolean = true): Unit = {
     var expected: Array[Row] = Array.empty
     var dfSpark: Dataset[Row] = null
     withSQLConf(CometConf.COMET_ENABLED.key -> "false", EXTENDED_EXPLAIN_PROVIDERS_KEY -> "") {
@@ -250,11 +251,17 @@ abstract class CometTestBase
     }
     val dfComet = Dataset.ofRows(spark, df.logicalPlan)
     checkAnswer(dfComet, expected)
-    val diff = StringUtils.difference(
-      dfSpark.queryExecution.explainString(ExtendedMode),
-      dfComet.queryExecution.explainString(ExtendedMode))
-    if (supportsExtendedExplainInfo(dfSpark.queryExecution)) {
-      assert(expectedInfo.forall(s => diff.contains(s)))
+    if (checkExplainString) {
+      val diff = StringUtils.difference(
+        dfSpark.queryExecution.explainString(ExtendedMode),
+        dfComet.queryExecution.explainString(ExtendedMode))
+      if (supportsExtendedExplainInfo(dfSpark.queryExecution)) {
+        for (info <- expectedInfo) {
+          if (!diff.contains(info)) {
+            fail(s"Extended explain diff did not contain [$info]. Diff: $diff.")
+          }
+        }
+      }
     }
     val extendedInfo =
       new ExtendedExplainInfo().generateExtendedInfo(dfComet.queryExecution.executedPlan)
@@ -422,6 +429,120 @@ abstract class CometTestBase
     makeParquetFileAllTypes(path, dictionaryEnabled, 0, n)
   }
 
+  def getPrimitiveTypesParquetSchema: String = {
+    if (CometSparkSessionExtensions.usingDataFusionParquetReader(conf)) {
+      // Comet complex type reader has different behavior for uint_8, uint_16 types.
+      // The issue stems from undefined behavior in the parquet spec and is tracked
+      // here: https://github.com/apache/parquet-java/issues/3142
+      // here: https://github.com/apache/arrow-rs/issues/7040
+      // and here: https://github.com/apache/datafusion-comet/issues/1348
+      if (isSpark34Plus) {
+        """
+         |message root {
+         |  optional boolean                  _1;
+         |  optional int32                    _2(INT_8);
+         |  optional int32                    _3(INT_16);
+         |  optional int32                    _4;
+         |  optional int64                    _5;
+         |  optional float                    _6;
+         |  optional double                   _7;
+         |  optional binary                   _8(UTF8);
+         |  optional int32                    _9(UINT_32);
+         |  optional int32                    _10(UINT_32);
+         |  optional int32                    _11(UINT_32);
+         |  optional int64                    _12(UINT_64);
+         |  optional binary                   _13(ENUM);
+         |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
+         |  optional int32                    _15(DECIMAL(5, 2));
+         |  optional int64                    _16(DECIMAL(18, 10));
+         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+         |  optional INT32                    _20(DATE);
+         |}
+        """.stripMargin
+      } else {
+        """
+         |message root {
+         |  optional boolean                  _1;
+         |  optional int32                    _2(INT_8);
+         |  optional int32                    _3(INT_16);
+         |  optional int32                    _4;
+         |  optional int64                    _5;
+         |  optional float                    _6;
+         |  optional double                   _7;
+         |  optional binary                   _8(UTF8);
+         |  optional int32                    _9(UINT_32);
+         |  optional int32                    _10(UINT_32);
+         |  optional int32                    _11(UINT_32);
+         |  optional int64                    _12(UINT_64);
+         |  optional binary                   _13(ENUM);
+         |  optional binary                   _14(UTF8);
+         |  optional int32                    _15(DECIMAL(5, 2));
+         |  optional int64                    _16(DECIMAL(18, 10));
+         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+         |  optional INT32                    _20(DATE);
+         |}
+        """.stripMargin
+      }
+    } else {
+
+      if (isSpark34Plus) {
+        """
+         |message root {
+         |  optional boolean                  _1;
+         |  optional int32                    _2(INT_8);
+         |  optional int32                    _3(INT_16);
+         |  optional int32                    _4;
+         |  optional int64                    _5;
+         |  optional float                    _6;
+         |  optional double                   _7;
+         |  optional binary                   _8(UTF8);
+         |  optional int32                    _9(UINT_8);
+         |  optional int32                    _10(UINT_16);
+         |  optional int32                    _11(UINT_32);
+         |  optional int64                    _12(UINT_64);
+         |  optional binary                   _13(ENUM);
+         |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
+         |  optional int32                    _15(DECIMAL(5, 2));
+         |  optional int64                    _16(DECIMAL(18, 10));
+         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+         |  optional INT32                    _20(DATE);
+         |}
+        """.stripMargin
+      } else {
+        """
+         |message root {
+         |  optional boolean                  _1;
+         |  optional int32                    _2(INT_8);
+         |  optional int32                    _3(INT_16);
+         |  optional int32                    _4;
+         |  optional int64                    _5;
+         |  optional float                    _6;
+         |  optional double                   _7;
+         |  optional binary                   _8(UTF8);
+         |  optional int32                    _9(UINT_8);
+         |  optional int32                    _10(UINT_16);
+         |  optional int32                    _11(UINT_32);
+         |  optional int64                    _12(UINT_64);
+         |  optional binary                   _13(ENUM);
+         |  optional binary                   _14(UTF8);
+         |  optional int32                    _15(DECIMAL(5, 2));
+         |  optional int64                    _16(DECIMAL(18, 10));
+         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+         |  optional INT32                    _20(DATE);
+         |}
+        """.stripMargin
+      }
+    }
+  }
+
   def makeParquetFileAllTypes(
       path: Path,
       dictionaryEnabled: Boolean,
@@ -429,58 +550,9 @@ abstract class CometTestBase
       end: Int,
       pageSize: Int = 128,
       randomSize: Int = 0): Unit = {
-    val schemaStr =
-      if (isSpark34Plus) {
-        """
-          |message root {
-          |  optional boolean                  _1;
-          |  optional int32                    _2(INT_8);
-          |  optional int32                    _3(INT_16);
-          |  optional int32                    _4;
-          |  optional int64                    _5;
-          |  optional float                    _6;
-          |  optional double                   _7;
-          |  optional binary                   _8(UTF8);
-          |  optional int32                    _9(UINT_8);
-          |  optional int32                    _10(UINT_16);
-          |  optional int32                    _11(UINT_32);
-          |  optional int64                    _12(UINT_64);
-          |  optional binary                   _13(ENUM);
-          |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
-          |  optional int32                    _15(DECIMAL(5, 2));
-          |  optional int64                    _16(DECIMAL(18, 10));
-          |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-          |  optional INT32                    _20(DATE);
-          |}
-        """.stripMargin
-      } else {
-        """
-          |message root {
-          |  optional boolean                  _1;
-          |  optional int32                    _2(INT_8);
-          |  optional int32                    _3(INT_16);
-          |  optional int32                    _4;
-          |  optional int64                    _5;
-          |  optional float                    _6;
-          |  optional double                   _7;
-          |  optional binary                   _8(UTF8);
-          |  optional int32                    _9(UINT_8);
-          |  optional int32                    _10(UINT_16);
-          |  optional int32                    _11(UINT_32);
-          |  optional int64                    _12(UINT_64);
-          |  optional binary                   _13(ENUM);
-          |  optional binary                   _14(UTF8);
-          |  optional int32                    _15(DECIMAL(5, 2));
-          |  optional int64                    _16(DECIMAL(18, 10));
-          |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-          |  optional INT32                    _20(DATE);
-          |}
-        """.stripMargin
-      }
+    // alwaysIncludeUnsignedIntTypes means we include unsignedIntTypes in the test even if the
+    // reader does not support them
+    val schemaStr = getPrimitiveTypesParquetSchema
 
     val schema = MessageTypeParser.parseMessageType(schemaStr)
     val writer = createParquetWriter(
